Attribute VB_Name = "modDrawText"
Option Explicit

Public Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Type DRAWTEXTPARAMS
    cbSize As Long
    iTabLength As Long
    iLeftMargin As Long
    iRightMargin As Long
    uiLengthDrawn As Long
End Type

Private Const LF_FACESIZE = 32
Private Type LOGFONT
    lfHeight As Long
    lfWidth As Long
    lfEscapement As Long
    lfOrientation As Long
    lfWeight As Long
    lfItalic As Byte
    lfUnderline As Byte
    lfStrikeOut As Byte
    lfCharSet As Byte
    lfOutPrecision As Byte
    lfClipPrecision As Byte
    lfQuality As Byte
    lfPitchAndFamily As Byte
    lfFaceName(LF_FACESIZE) As Byte
End Type

Private Declare Function CreateFontIndirect Lib "gdi32" Alias "CreateFontIndirectA" (lpLogFont As LOGFONT) As Long
Private Declare Function DrawTextEx Lib "user32" Alias "DrawTextExA" (ByVal hdc As Long, ByVal lpsz As String, ByVal n As Long, lpRect As RECT, ByVal un As Long, lpDrawTextParams As DRAWTEXTPARAMS) As Long
Private Declare Function SetTextColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
Private Declare Function FillRect Lib "user32" (ByVal hdc As Long, lpRect As RECT, ByVal hBrush As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long


Private Const DEFAULT_QUALITY = 0
Private Const DEFAULT_PITCH = 0
Private Const DEFAULT_CHARSET = 1
Private Const OUT_DEFAULT_PRECIS = 0
' DrawText Flags
Public Const DT_BOTTOM = &H8
Public Const DT_CALCRECT = &H400
Public Const DT_CENTER = &H1
Public Const DT_CHARSTREAM = 4
Public Const DT_DISPFILE = 6
Public Const DT_EXPANDTABS = &H40
Public Const DT_EXTERNALLEADING = &H200
Public Const DT_LEFT = &H0
Public Const DT_METAFILE = 5
Public Const DT_NOCLIP = &H100
Public Const DT_NOPREFIX = &H800
Public Const DT_PLOTTER = 0
Public Const DT_RASCAMERA = 3
Public Const DT_RASDISPLAY = 1
Public Const DT_RASPRINTER = 2
Public Const DT_RIGHT = &H2
Public Const DT_SINGLELINE = &H20
Public Const DT_TOP = &H0
Public Const DT_VCENTER = &H4
Public Const DT_WORDBREAK = &H10
Public Const DT_END_ELLIPSIS = &H8000


' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   DrawText: Draws the text to a device context
'       Parameters: strText, the Text to draw
'                   hDc, the device context to draw to
'                   udtRect, the region with the device context
'                   lngForeColor, the color of the text
'                   lngBackColor, the color of the background
'
'       NOTE : The default for the lngForeColor, lngBackColor is .5 because
'           Visual Basic can not determine if the parameter is missing on a
'           long
'
'       Returns:  the return code from DrawTextEx
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Public Function DrawText(strText As String, _
                        hdc As Long, _
                        objFont As StdFont, _
                        ByRef udtRect As RECT, _
                        Optional ForeColor As Variant, _
                        Optional BackColor As Variant, _
                        Optional Angle As Variant = 0, _
                        Optional Flags As Variant = DT_END_ELLIPSIS _
                        ) As Long

    
    Dim lf              As LOGFONT      ' The Font Structure
    Dim lngFontToUse    As Long         ' a pointer to the new font
    Dim lngOldHdc       As Long         ' the old hdc
    Dim TempByteArray() As Byte
    Dim rc              As RECT
    Dim intByteArrayLimit   As Integer
    Dim intCounter      As Integer
    Dim lngReturn       As Long
    
    Dim lngForeColor    As Long
    Dim lngBackColor    As Long
    Dim lngAngle        As Long
    Dim lngFlags        As Long
    
    Dim udtDrawText     As DRAWTEXTPARAMS
    Dim hBrushNew       As Long
    
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    '   Compansate for "Missing" Arguements
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    On Error Resume Next
    lngAngle = Angle
    lngFlags = Flags
    lngForeColor = CLng(ForeColor)
    lngBackColor = CLng(BackColor)
    On Error GoTo 0
    
    With udtDrawText
        .cbSize = Len(udtDrawText)
        .uiLengthDrawn = Len(strText)
    End With
    
    On Error Resume Next
    ' If a font was not specified use the default for the Hdc
    lf.lfQuality = 0
    
    If Err.Number Then GoTo DrawTextNow
    
    On Error GoTo 0
    
    
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    '   Set the Text & background color
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    If Not IsMissing(ForeColor) Then
        lngReturn = SetTextColor(hdc, lngForeColor)
    End If
    
    If Not IsMissing(BackColor) Then
        lngReturn = SetBkColor(hdc, lngBackColor)

        ' Create a brush with the new background color
        hBrushNew = CreateSolidBrush(lngBackColor)
        ' Set the Background to that of the appropriate color (indicate whether the
        '   item is selected or not)
        lngReturn = FillRect(hdc, udtRect, hBrushNew)
        lngReturn = DeleteObject(hBrushNew)
    End If

    
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    '   Create the Font
    ' @~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~@
    With lf
        .lfEscapement = 0
        .lfHeight = objFont.Size
        .lfItalic = objFont.Italic
        .lfStrikeOut = objFont.Strikethrough
        .lfUnderline = objFont.Underline
        .lfWeight = objFont.Weight
        .lfOutPrecision = OUT_DEFAULT_PRECIS
        .lfClipPrecision = OUT_DEFAULT_PRECIS
        .lfQuality = DEFAULT_QUALITY
        .lfPitchAndFamily = DEFAULT_PITCH
        .lfCharSet = DEFAULT_CHARSET
    End With
    
    ' Convert into a Byte Arrary
    TempByteArray = StrConv(objFont.Name & Chr$(0), vbFromUnicode)
    
    intByteArrayLimit = UBound(TempByteArray)
    
    For intCounter = 0 To intByteArrayLimit
        lf.lfFaceName(intCounter) = TempByteArray(intCounter)
    Next intCounter
    
    lngFontToUse = CreateFontIndirect(lf)
    
    If lngFontToUse = 0 Then GoTo DrawTextNow
    
    lngOldHdc = SelectObject(hdc, lngFontToUse)
    
    
DrawTextNow:
    On Error GoTo 0
    DrawText = DrawTextEx(hdc, strText, Len(strText), udtRect, lngFlags, udtDrawText)
    
    
    Call SelectObject(hdc, lngOldHdc)
    Call DeleteObject(lngFontToUse)
End Function


